import io
import json
import logging

import pandas

from sdf.utils import get_output_path, get_time, process, custom_json_dump


class GCP_SDF:
    def __init__(self, config, blob, storage_client, bigquery_client):
        self.config = config
        self.input_path = config["input_path"]
        self.output_path = config["output_path"]
        self.bucket_name = config["bucket_name"]
        self.blob = blob

        self.table_name = config["reconciliation"]
        self.src = "gcs"

        self.storage_client = storage_client
        self.bigquery_client = bigquery_client
        self.processed_data = None

    def update_storage(self):
        """Stores the error into sink"""
        bucket = self.storage_client.get_bucket(self.bucket_name)
        if self.blob is None:
            logging.error(f"input_path: {self.input_path} does not exist")
            return

        file_contents = self.blob.download_as_string()

        metadata = {
            "_rt": self.received_timestamp,
            "_src": self.src,
            "_o": "",
            "src_dtls": self.blob.public_url,
        }

        self.processed_data = process(file_contents, metadata)

        # Create destination blob
        dest_blob = bucket.blob(get_output_path(self.blob.name, self.output_path))
        dest_blob.upload_from_string(
            custom_json_dump(self.processed_data), content_type="application/json"
        )
        return True

    def update_table(self):
        """Update table with data"""
        data = [
            {
                "src": self.src,
                "src_dtls": self.blob.public_url,
                "record_count": len(self.processed_data),
                "received_timestamp": self.received_timestamp,
                "processed_timestamp": get_time(),
            }
        ]
        print("Inserting data into recon table")
        self.bigquery_client.insert_rows_json(self.table_name, data)

    def run(self):
        """Entrypoint"""
        res = self.update_storage()
        if res:
            self.update_table()

    @property
    def received_timestamp(self):
        """Convert timestamp to string"""
        return self.blob.time_created.strftime("%Y-%m-%d %X")



import io
import json
import logging
from os.path import join

import pandas
from botocore.exceptions import ClientError


from sdf.utils import get_output_path, get_time, process, custom_json_dump


class AWS_SDF:
    def __init__(self, config, blob, s3_resource, s3_client):
        self.config = config
        self.input_path = config["input_path"]
        self.output_path = config["output_path"]
        self.bucket_name = config["bucket_name"]
        self.blob = blob

        self.src = "aws"

        self.s3_resource = s3_resource
        self.s3_client = s3_client
        self.bucket = self.s3_resource.Bucket(self.bucket_name)
        self.processed_data = None

    def update_storage(self):
        """Stores the error into sink"""
       
        file_contents = self.blob.get()['Body'].read()

        metadata = {
            "_rt": self.received_timestamp,
            "_src": self.src,
            "_o": "",
            "src_dtls": self.src_details,
        }
        self.processed_data = process(file_contents, metadata)
        self.bucket.put_object(Body=custom_json_dump(self.processed_data), Key=get_output_path(self.blob._key, self.output_path))
        return True

    def update_table(self):
        """Update table with data"""
        data = [
            {
                "src": self.src,
                "src_dtls": self.src_details,
                "record_count": len(self.processed_data),
                "received_timestamp": self.received_timestamp,
                "processed_timestamp": get_time(),
            }
        ]
        data_df = pandas.DataFrame(data)
        recon = None
        existing_csv_data = None
        key = self.config.get("reconciliation")
        try:
            recon = self.s3_resource.Object(self.bucket_name, key)
            existing_csv_data = recon.get()['Body']
        except ClientError as e:
            if e.response['Error']['Code'] == 'NoSuchKey':
                pass
            else:
                raise

        if existing_csv_data is None:
            recon.put(Body=data_df.to_csv(index=False).encode())
        else:
            recon.put(Body=pandas.read_csv(existing_csv_data).append(data_df).to_csv(index=False).encode())

    def run(self):
        """Entrypoint"""
        res = self.update_storage()
        if res:
            self.update_table()

    @property
    def received_timestamp(self):
        """Convert timestamp to string"""
        return self.blob.last_modified.strftime("%Y-%m-%d %X")

    @property
    def src_details(self):
        return self.blob._bucket_name + '/' + self.blob._key



import csv
import io
import json
import os
import sys
import pandas
from datetime import datetime


def get_config(path):
    if not os.path.exists(path):
        print("config file not found")
        return False

    with open(path, "r") as f:
        return json.load(f)


def get_filename(path):
    return os.path.splitext(os.path.basename(path))[0]


def get_output_path(blob_name, output_path):
    filename = get_filename(blob_name)
    op_path = os.path.join(output_path, f"{filename}.json").replace("\\", "/")
    print(f"Writing to {op_path}")
    return op_path


def get_time():
    now = datetime.now()
    return now.strftime("%Y-%m-%d %X")

def get_fields(data):
    return data.splitlines()[0].decode("utf-8").split(",")


def process(data, metadata):
    """Adds metadata tags"""
    parsed_data = pandas.read_csv(io.StringIO(data.decode("utf-8")))

    updated_data = list()
    for _, row in parsed_data.iterrows():
        updated_data.append({"_m": metadata, "_p": {"data": dict(row)}})
    return updated_data

def custom_json_dump(processed_data):
    """
    Takes in processed data and returns a custom JSON format with
    each line representing valid JSON.
    """
    return "\n".join(list(map(json.dumps,  processed_data)))

class Cloud:
    AWS = "AWS"
    GCP = "GCP"
    AZURE = "AZURE"



import os
import sys
import json

from google.cloud import storage, bigquery
import boto3
from sdf.aws_sdf import AWS_SDF

from sdf.utils import get_config, Cloud
from sdf.gcp_sdf import GCP_SDF


def main():
    # Check if env variable is set
    config = get_config(sys.argv[1])
    cloud = config.get("cloud")

    if not os.environ.get("GOOGLE_APPLICATION_CREDENTIALS") and cloud == Cloud.GCP:
        print("Please set 'GOOGLE_APPLICATION_CREDENTIALS' environment variable")
        os._exit(-1)

    if len(sys.argv) < 2:
        print("Provide path to config.json as argument.")
        os._exit(-1)

    if cloud == Cloud.AWS:
        aws_main(config)
    elif cloud == Cloud.GCP:
        gcp_main(config)
    elif cloud == Cloud.AZURE:
        raise NotImplementedError("Planned for future")


def aws_main(config):
    client = boto3.client('s3')
    s3_resource = boto3.resource('s3')

    bucket_name = config["bucket_name"]
    objects = client.list_objects_v2(
        Bucket=bucket_name,
        Delimiter="/",
        Prefix=config["input_path"]
    )

    contents = objects.get("Contents")
    if contents is None:
        print("Provided bucket_name and input_path contain no objects", file=sys.stderr)
        return

    contents = list(filter(lambda x: x.get("Size") != 0, contents))
    print(f"Found {len(contents)} blobs. Processing...")

    for index, blob in enumerate(contents):
        key = blob.get("Key")
        print(f"Processing {index + 1} of {len(contents)}: {key}")
        s3_object = s3_resource.Object(bucket_name, key)
        sdf = AWS_SDF(config, s3_object, s3_resource, client)
        sdf.run()


def gcp_main(config):
    storage_client = storage.Client()
    bigquery_client = bigquery.Client()

    all_blobs = list(storage_client.list_blobs(
        bucket_or_name=config["bucket_name"],
        prefix=config["input_path"])
    )
    print(f"Found {len(all_blobs)} blobs. Processing...")

    for index, blob in enumerate(all_blobs):
        print(f"Processing {index + 1} of {len(all_blobs)}: {blob.name}")
        sdf = GCP_SDF(config, blob, storage_client, bigquery_client)
        sdf.run()


if __name__ == "__main__":
    main()



import setuptools
import os

local_path = os.path.dirname(__file__)
# Fix for tox which manipulates execution pathing
if not local_path:
    local_path = '.'
here = os.path.abspath(local_path)


def read(fname):
    with open(fname, 'r') as fhandle:
        return fhandle.read()


def version():
    with open(here + '/sdf/_version.py', 'r') as ver:
        for line in ver.readlines():
            if line.startswith('version ='):
                return line.split(' = ')[-1].strip()[1:-1]
    raise ValueError('No version found in sdf/_version.py')


with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

requirements = read(os.path.join(os.path.dirname(__file__), "requirements.txt"))


setuptools.setup(
    name="sankir-sdf",
    version=version(),
    author="SanKir Technologies Pvt Ltd",
    author_email="sanjay@sankir.com",
    description="SDF Converter",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/SanKirTech/sdf-converter",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    entry_points={
        "console_scripts": [
            "sankir-sdf=sdf.__main__:main"
        ]
    },
    install_requires=requirements,
    python_requires='>=3.6',
)